;; This is stolen from hilit-lookup-face-create.  I modified it to be a bit
;; more convenient - use "#" and "h-" instead of "hex-" and "*" instead of
;; "default".

(defun lookup-face-create (face &optional force)
  "Get a FACE, or create it if it doesn't exist.
In order for it to properly create the face, the following naming convention
must be used:
    (fgcolor[/bgcolor])[-bold][-italic][-underline]
Example: (lookup-face-create yellow/blue4-bold) might create and return an
appropriate face.

Each color is either the name of an X color (see .../X11/lib/X11/rgb.txt),
a hexadecimal specification of the form \"hex-[0-9A-Fa-f]+\" (\"hex-\" can be
replaced by \"h-\" or \"#\"), or \"default\".

An optional argument, FORCE, will cause the face to be recopied from the
default...which is probably of use only if you've changed fonts."

  ;;Eli: if no colors - return default face.
  (if (not window-system)
    'default

    ;; make the face if we need to...
    (let* ((fn       (symbol-name face))
           (frame    (selected-frame))
           (basefont (cdr (assq 'font (frame-parameters frame))))
           error fgcolor bgcolor)
      (cond
        ((or (null face) (memq face (face-list)))
         ;; do nothing if the face is nil or if it's predefined.
         )
        ((or force
             (not (memq face (face-list)))
             (not (string= (get face 'basefont) basefont)))
         (copy-face 'default 'scratch-face)
         ;; parse foreground color
         (if (string-match "^\\(hex-\\|h-\\|#\\)?\\([A-Za-z0-9*]+\\)" fn)
           (setq fgcolor (concat
                          (if (match-beginning 1) "#")
                          (substring fn (match-beginning 2) (match-end 2)))
                 fn (substring fn (match-end 0)))
           (error "Bad face name %S" face))
         ;; parse background color
         (if (string-match "^/\\(hex-\\|h-\\|#\\)?\\([A-Za-z0-9*]+\\)" fn)
           (setq bgcolor (concat
                          (if (match-beginning 1) "#")
                          (substring fn (match-beginning 2) (match-end 2)))
                 fn (substring fn (match-end 0))))
         (if (or (string= "*" fgcolor) (string= "default" fgcolor))
           (setq fgcolor nil))
         (if (or (string= "*" bgcolor) (string= "default" bgcolor))
           (setq bgcolor nil))
         ;; catch errors if we can't allocate the color(s)
         (condition-case nil
             (progn (set-face-foreground 'scratch-face fgcolor)
                    (set-face-background 'scratch-face bgcolor)
                    (copy-face 'scratch-face face)
                    (put face 'basefont basefont))
           (error (message "couldn't allocate color for '%s'"
                           (symbol-name face))
                  (setq face 'default)
                  (setq error t)))
         (if (not error)
           ;; don't bother w/ bold or italic if we didn't get the color
           ;; we wanted, but ignore errors making the face bold or italic
           ;; if the font isn't available, there's nothing to do about it...
           (progn
             (set-face-font face nil frame)
             (set-face-underline-p face (string-match "underline" fn))
             (if (string-match "-bold" fn)
               ;; make face bold in all frames
               (make-face-bold face nil 'noerr))
             (if (string-match "-italic" fn)
               ;; make face italic in all frames
               (make-face-italic face nil 'noerr))
             ))))
      face)))

;; This is a cheap small function - not using overlays or removing faces etc.
;; A better version may be facemenu-add-face or hilit-region-set-face.
(defun set-region-face (beg end face &optional no-create)
  "Set the face of a region.
Set the region from BEG to END to have face FACE.  `lookup-face-create' is used
to create the face, unless NO-CREATE is t."
  (and
   window-system
   (let (;; These are taken from font-lock.el - make sure that we modify the
         ;; buffer with absolutely no trace.
         (modified (buffer-modified-p))
         (buffer-undo-list t)
         before-change-functions after-change-functions
         buffer-file-name buffer-file-truename)
     (put-text-property beg end 'face
                        (if no-create face (lookup-face-create face)))
     (and (not modified) (buffer-modified-p) (set-buffer-modified-p nil)))))
;; Using the above function.
(defun set-regexp-face (regexp face &optional subexp)
  "Set the face of all occurrences of REGEXP to FACE.
Optional argument SUBEXP is either a subexpression to highlight, or a list of
pairs of subexp number and its color: (SUB . COLOR)."
  (and window-system
       (let ((face (and face (lookup-face-create face))))
         (if (consp subexp)
           (let ((p subexp))
             (while p
               (setcdr (car p) (lookup-face-create (cdr (car p))))
               (setq p (cdr p)))
             (and face (setq subexp (cons (cons 0 face) subexp)))))
         (save-excursion
           (goto-char (point-min))
           (while (re-search-forward regexp nil t nil)
             (cond
               ((consp subexp)
                (let ((p subexp))
                  (while p
                    (set-region-face (match-end (car (car p)))
                                     (match-beginning (car (car p)))
                                     (cdr (car p)) t)
                    (setq p (cdr p)))))
               (face
                (set-region-face (match-end (or subexp 0))
                                 (match-beginning (or subexp 0))
                                 face t))))))))
