;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; misc lisp functions

;;;
(defun select-all ()
  (interactive)
  (set-mark (point-min))
  (goto-char (point-max)))
;;;

;;;
(defadvice previous-history-element (after end-of-buffer activate)
  "Goto the end of the minibuffer."
  (goto-char (point-max)))
;;;

;;;
(defadvice next-history-element (after end-of-buffer activate)
  "Goto the end of the minibuffer."
  (goto-char (point-max)))
;;;

;;; Either indent according to mode, or expand the word preceding point.
(defun indent-or-expand () 
  "Either indent according to mode, or expand the word preceding point."
  (interactive)
  (if (and
  (= ?w (char-syntax (char-before)))
  (not (= ?w (char-syntax (char-after)))))
      (unwind-protect (dabbrev-expand nil))
    (indent-according-to-mode)))
;;;

;;; Switch to the next buffer, skipping non-file buffers
(defun next-buffer ()
  (interactive)
  (bury-buffer)
  (setq bufname (buffer-name))
  (if (or (or (string-equal bufname "*scratch*")
	      (string-equal bufname "*Messages*"))
  (or (string-equal bufname "*Apropos*")
      (string-equal bufname "*Completions*")))
      (bury-buffer)
    (if (or (or (string-equal bufname "*Help*")
		(string-equal bufname "*Buffer List*"))
	    (string-equal bufname "*compilation*"))
    (bury-buffer))))
;;;

;;; kills all back-up files
(defun kill-all-backup-files ()
  (interactive)
  (let ((bufs (buffer-list)))
    (while bufs
    (let ((file (buffer-file-name (car bufs))))
      (when file
	(let* ((dir (file-name-directory file))
	       (dired-buf (find-file-noselect dir)))
	  (set-buffer dired-buf)
	  (dired-unmark-all-marks)
	  (dired-flag-backup-files)
	  (let ((dired-no-confirm t)
		(dired-deletion-confirmer #'(lambda (&rest args) t)))
	    (dired-do-flagged-delete t)))))
    (setq bufs (cdr bufs)))))
(add-hook 'kill-emacs-hook 'kill-all-backup-files)
;;;

;;;
(defun previous-window (arg)
  "Switch to previous window."
  (interactive "p")
  (other-window (- arg)))
;;;

;;; call the align.el package and align region as one block
(defun my-align-entire (pt mk)
  (interactive "r")
  ;;(align pt mk "entire"))
  (align pt mk "group"))
;;;

;;; after uncommenting a region, also re-align it.
(defun my-uncomment-and-align (pt mk)
  (interactive "r")
  (save-excursion
    (comment-out-region pt mk -2)
    ;; (my-align-entire pt mk)
    ))
;;;

;;; Jump to top of current page
(defun joc-top-of-page ()
  "Brings point to the top of the current window"
  (interactive)
  (move-to-window-line 0))
;;;

;;; Jump to bottom of current page
(defun joc-bottom-of-page ()
  "Brings point to the bottom of the current window"
  (interactive)
  (move-to-window-line -1))
;;;

;;; Force the showing of matching paren
(defun match-paren (arg)
  "Go to the matching parenthesis if on parenthesis otherwise insert %."
  (interactive "p")
  (cond ((looking-at "\\s\(") (forward-list 1) (backward-char 1))
	((looking-at "\\s\)") (forward-char 1) (backward-list 1))
	(t (self-insert-command (or arg 1)))))
;;;

;;; Make a window split show current buffer and scratch
(defun my-split-different-windows ()
  (interactive)
  (split-window)
  (set-window-buffer (next-window (selected-window))
                     (get-buffer-create "*scratch*"))
  (other-window 1))
;;;

;;; Open my .emacs file
(defun my-load-DOTemacs ()
  (interactive)
  (folding-mode t)
  (find-file "~/.emacs"))
  ;; (find-file "/afs/wam.umd.edu/home/wam/j/o/joviedo/home/.emacs"))
  ;; (find-file "c:/_emacs"))
;;;

;;; Load dired with the contents of my site-lisp directory
(defun my-site-lisp-load ()
  (interactive)
  (find-file "~/site-lisp/"))
  ;; (find-file "/afs/wam.umd.edu/home/wam/j/o/joviedo/home/site-lisp/"))
  ;; (find-file "c:/joviedo/emacs-20.7/site-lisp/"))
;;;

;;; Kill buffers by mode, using a regexp
(defun kill-buffers-by-mode (regex)
  "Kill buffers whose major mode matches regexp"
  (interactive "sMajor mode regexp: ")
  (let ((buflist (buffer-list))
  buf)
    (while buflist
    (setq buf (car buflist))
    (setq buflist (cdr buflist))
    (set-buffer buf)
    (if (string-match regex mode-name)
	(kill-buffer buf)))))
;;;

;;; Duplicate current line and paste it in the next line.
(defun dup-line()
  "Copy the current line and insert copy as the following line."
  (interactive)                            
  (save-excursion
    (forward-line 1)
    (copy-from-above-command)
    (insert "\n")))
;;;


;;; Inserts time stamp modification on files that have been changed
;;; on disk
(defun my-time-stamp ()
  (interactive)
  (save-excursion
    (let ()
    (goto-char (point-min))
    (if (eq major-mode 'emacs-lisp-mode)
	(let ()
	  (replace-regexp
	   "LAST[^/]+Oviedo\n"
	   "LAST MODIFIED ON:  \n;;; LAST MODIFIED BY:  Javier Oviedo\n")
	  (let ()
	    (backward-word 6)
	    (forward-char 5)
	    (insert(format-time-string "%a %d-%b-%Y %T")))))))
  (save-buffer))
;;;

;;; Change the time stamp of the current buffer if the buffer
;;; was modified.
(defun my-save-buffer ()
  (interactive)
  (if (buffer-modified-p)
      (my-time-stamp))
  (save-buffer))
;;;

;;; After pasting code automatically indent it.
(defun yank-and-indent ()
  "yank and then indent the newly formed region according to mode"
  (interactive)
  (yank)
  (let* ((my-reg-beg (region-beginning))
  (my-reg-end (region-end))
  (my-point my-reg-beg))
    (save-excursion
    (goto-char my-reg-beg)
    (while (<= (point) my-reg-end)
      (indent-according-to-mode)
      (next-line 1)))))
;;;


;;; end - misc lisp functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(provide 'misc-lisp-functions)
